@@ -12,6 +12,32 @@
  *  You should have received a copy of the GNU Affero General Public License along with this program.  If not, see   *
 \*  <http://www.gnu.org/licenses/>.                                                                                  */
 
+
+impl MedalObject<Connection> for Submission {
+    fn save(&mut self, conn: &Connection) {
+        match self.get_id() {
+            Some(_id) => unimplemented!(),
+            None => {
+                let query = "INSERT INTO submission (task, session, grade, validated, nonvalidated_grade,
+                                                     subtask_identifier, value, date, needs_validation)
+                             VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9)";
+                conn.execute(query,
+                             &[&self.task,
+                               &self.session_user,
+                               &self.grade,
+                               &self.validated,
+                               &self.nonvalidated_grade,
+                               &self.subtask_identifier,
+                               &self.value,
+                               &self.date,
+                               &self.needs_validation])
+                    .unwrap();
+                self.set_id(conn.get_last_id().unwrap());
+            }
+        }
+    }
+}
+
 impl MedalObject<Connection> for Participation {
     fn save(&mut self, conn: &Connection) {
         let query = "INSERT INTO participation (contest, session, start_date)
@@ -125,8 +151,9 @@ impl MedalObject<Connection> for Contest {
             Some(id) => {
                 let query = "UPDATE contest
                              SET location = $2,filename = $3, name = $4, duration = $5, public = $6, start_date = $7,
-                                 end_date = $8, min_grade = $9, max_grade = $10, positionalnumber = $11,
-                                 requires_login = $12, requires_contest = $13, secret = $14, message = $15
+                                 end_date = $8, review_start_date = $9, review_end_date = $10, min_grade = $11,
+                                 max_grade = $12, positionalnumber = $13, requires_login = $14, requires_contest = $15,
+                                 secret = $16, message = $17
                              WHERE id = $1";
                 conn.execute(query,
                              &[&id,
@@ -137,6 +164,8 @@ impl MedalObject<Connection> for Contest {
                                &self.public,
                                &self.start,
                                &self.end,
+                               &self.review_start,
+                               &self.review_end,
                                &self.min_grade,
                                &self.max_grade,
                                &self.positionalnumber,
@@ -149,9 +178,9 @@ impl MedalObject<Connection> for Contest {
             }
             None => {
                 let query = "INSERT INTO contest (location, filename, name, duration, public, start_date, end_date,
-                                                  min_grade, max_grade, positionalnumber, requires_login,
-                                                  requires_contest, secret, message)
-                             VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14)";
+                                                  review_start_date, review_end_date, min_grade, max_grade,
+                                                  positionalnumber, requires_login, requires_contest, secret, message)
+                             VALUES ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10, $11, $12, $13, $14, $15, $16)";
                 conn.execute(query,
                              &[&self.location,
                                &self.filename,
@@ -160,6 +189,8 @@ impl MedalObject<Connection> for Contest {
                                &self.public,
                                &self.start,
                                &self.end,
+                               &self.review_start,
+                               &self.review_end,
                                &self.min_grade,
                                &self.max_grade,
                                &self.positionalnumber,
@@ -244,15 +275,18 @@ impl MedalConnection for Connection {
                                                                              oauth_foreign_id: row.get(24) })
                           .ok()??;
 
-        let duration = Duration::hours(12);
+        let session_duration = Duration::hours(12);
+        let mimimal_activity_update_duration = Duration::minutes(3);
         let now = time::get_time();
 
         if let Some(last_activity) = session.last_activity {
-            if now - last_activity < duration {
-                let query = "UPDATE session
-                             SET last_activity = $1
-                             WHERE id = $2";
-                self.execute(query, &[&now, &session.id]).unwrap();
+            if now < last_activity + session_duration {
+                if now > last_activity + mimimal_activity_update_duration {
+                    let query = "UPDATE session
+                                 SET last_activity = $1
+                                 WHERE id = $2";
+                    self.execute(query, &[&now, &session.id]).unwrap();
+                }
                 return Some(session);
             } else {
                 // Session timed out
@@ -1001,8 +1035,9 @@ impl MedalConnection for Connection {
     }
 
     fn get_contest_list(&self) -> Vec<Contest> {
-        let query = "SELECT id, location, filename, name, duration, public, start_date, end_date, min_grade, max_grade,
-                            positionalnumber, requires_login, requires_contest, secret, message
+        let query = "SELECT id, location, filename, name, duration, public, start_date, end_date, review_start_date,
+                            review_end_date, min_grade, max_grade, positionalnumber, requires_login, requires_contest,
+                            secret, message
                      FROM contest
                      ORDER BY positionalnumber DESC";
         self.query_map_many(query, &[], |row| Contest { id: Some(row.get(0)),
@@ -1013,20 +1048,22 @@ impl MedalConnection for Connection {
                                                         public: row.get(5),
                                                         start: row.get(6),
                                                         end: row.get(7),
-                                                        min_grade: row.get(8),
-                                                        max_grade: row.get(9),
-                                                        positionalnumber: row.get(10),
-                                                        requires_login: row.get(11),
-                                                        requires_contest: row.get(12),
-                                                        secret: row.get(13),
-                                                        message: row.get(14),
+                                                        review_start: row.get(8),
+                                                        review_end: row.get(9),
+                                                        min_grade: row.get(10),
+                                                        max_grade: row.get(11),
+                                                        positionalnumber: row.get(12),
+                                                        requires_login: row.get(13),
+                                                        requires_contest: row.get(14),
+                                                        secret: row.get(15),
+                                                        message: row.get(16),
                                                         taskgroups: Vec::new() })
             .unwrap()
     }
 
     fn get_contest_by_id(&self, contest_id: i32) -> Contest {
-        let query = "SELECT location, filename, name, duration, public, start_date, end_date, min_grade, max_grade,
-                            requires_login, requires_contest, secret, message
+        let query = "SELECT location, filename, name, duration, public, start_date, end_date, review_start_date,
+                            review_end_date, min_grade, max_grade, requires_login, requires_contest, secret, message
                      FROM contest
                      WHERE id = $1";
         self.query_map_one(query, &[&contest_id], |row| Contest { id: Some(contest_id),
@@ -1037,13 +1074,15 @@ impl MedalConnection for Connection {
                                                                   public: row.get(4),
                                                                   start: row.get(5),
                                                                   end: row.get(6),
-                                                                  min_grade: row.get(7),
-                                                                  max_grade: row.get(8),
+                                                                  review_start: row.get(7),
+                                                                  review_end: row.get(8),
+                                                                  min_grade: row.get(9),
+                                                                  max_grade: row.get(10),
                                                                   positionalnumber: None,
-                                                                  requires_login: row.get(9),
-                                                                  requires_contest: row.get(10),
-                                                                  secret: row.get(11),
-                                                                  message: row.get(12),
+                                                                  requires_login: row.get(11),
+                                                                  requires_contest: row.get(12),
+                                                                  secret: row.get(13),
+                                                                  message: row.get(14),
                                                                   taskgroups: Vec::new() })
             .unwrap()
             .unwrap() // TODO: Should return Option?
@@ -1051,8 +1090,9 @@ impl MedalConnection for Connection {
 
     fn get_contest_by_id_complete(&self, contest_id: i32) -> Contest {
         let query = "SELECT contest.location, contest.filename, contest.name, contest.duration, contest.public,
-                            contest.start_date, contest.end_date, contest.min_grade, contest.max_grade,
-                            contest.requires_login, contest.requires_contest, contest.secret, contest.message,
+                            contest.start_date, contest.end_date, contest.review_start_date, contest.review_end_date,
+                            contest.min_grade, contest.max_grade, contest.requires_login, contest.requires_contest,
+                            contest.secret, contest.message,
                             taskgroup.id, taskgroup.name,
                             task.id, task.location, task.stars
                      FROM contest
@@ -1071,21 +1111,23 @@ impl MedalConnection for Connection {
                                public: row.get(4),
                                start: row.get(5),
                                end: row.get(6),
-                               min_grade: row.get(7),
-                               max_grade: row.get(8),
+                               review_start: row.get(7),
+                               review_end: row.get(8),
+                               min_grade: row.get(9),
+                               max_grade: row.get(10),
                                positionalnumber: None,
-                               requires_login: row.get(9),
-                               requires_contest: row.get(10),
-                               secret: row.get(11),
-                               message: row.get(12),
+                               requires_login: row.get(11),
+                               requires_contest: row.get(12),
+                               secret: row.get(13),
+                               message: row.get(14),
                                taskgroups: Vec::new() },
-                     Taskgroup { id: Some(row.get(13)),
+                     Taskgroup { id: Some(row.get(15)),
                                  contest: contest_id,
-                                 name: row.get(14),
+                                 name: row.get(16),
                                  active: true,
                                  positionalnumber: None,
                                  tasks: Vec::new() },
-                     Task { id: Some(row.get(15)), taskgroup: row.get(13), location: row.get(16), stars: row.get(17) })
+                     Task { id: Some(row.get(17)), taskgroup: row.get(15), location: row.get(18), stars: row.get(19) })
                 })
                 .unwrap();
         let mut taskgroupcontest_iter = taskgroupcontest.into_iter();
@@ -1111,8 +1153,9 @@ impl MedalConnection for Connection {
 
     fn get_contest_by_id_partial(&self, contest_id: i32) -> Contest {
         let query = "SELECT contest.location, contest.filename, contest.name, contest.duration, contest.public,
-                            contest.start_date, contest.end_date, contest.min_grade, contest.max_grade,
-                            contest.requires_login, contest.requires_contest, contest.secret, contest.message,
+                            contest.start_date, contest.end_date, contest.review_start_date, contest.review_end_date,
+                            contest.min_grade, contest.max_grade, contest.requires_login, contest.requires_contest,
+                            contest.secret, contest.message,
                             taskgroup.id, taskgroup.name
                      FROM contest
                      JOIN taskgroup ON contest.id = taskgroup.contest
@@ -1127,17 +1170,19 @@ impl MedalConnection for Connection {
                                                   public: row.get(4),
                                                   start: row.get(5),
                                                   end: row.get(6),
-                                                  min_grade: row.get(7),
-                                                  max_grade: row.get(8),
+                                                  review_start: row.get(7),
+                                                  review_end: row.get(8),
+                                                  min_grade: row.get(9),
+                                                  max_grade: row.get(10),
                                                   positionalnumber: None,
-                                                  requires_login: row.get(9),
-                                                  requires_contest: row.get(10),
-                                                  secret: row.get(11),
-                                                  message: row.get(12),
+                                                  requires_login: row.get(11),
+                                                  requires_contest: row.get(12),
+                                                  secret: row.get(13),
+                                                  message: row.get(14),
                                                   taskgroups: Vec::new() },
-                                        Taskgroup { id: Some(row.get(13)),
+                                        Taskgroup { id: Some(row.get(15)),
                                                     contest: contest_id,
-                                                    name: row.get(14),
+                                                    name: row.get(16),
                                                     active: true,
                                                     positionalnumber: None,
                                                     tasks: Vec::new() })
@@ -1179,8 +1224,8 @@ impl MedalConnection for Connection {
 
     fn get_all_participations_complete(&self, session_id: i32) -> Vec<(Participation, Contest)> {
         let query = "SELECT participation.start_date, contest.id, location, filename, name, duration, public,
-                            contest.start_date, end_date, min_grade, max_grade, requires_login, requires_contest,
-                            secret, message
+                            contest.start_date, end_date, review_start_date, review_end_date, min_grade, max_grade,
+                            requires_login, requires_contest, secret, message
                      FROM participation
                      JOIN contest ON participation.contest = contest.id
                      WHERE participation.session = $1";
@@ -1194,13 +1239,15 @@ impl MedalConnection for Connection {
                            public: row.get(6),
                            start: row.get(7),
                            end: row.get(8),
-                           min_grade: row.get(9),
-                           max_grade: row.get(10),
+                           review_start: row.get(9),
+                           review_end: row.get(10),
+                           min_grade: row.get(11),
+                           max_grade: row.get(12),
                            positionalnumber: None,
-                           requires_login: row.get(11),
-                           requires_contest: row.get(12),
-                           secret: row.get(13),
-                           message: row.get(14),
+                           requires_login: row.get(13),
+                           requires_contest: row.get(14),
+                           secret: row.get(15),
+                           message: row.get(16),
                            taskgroups: Vec::new() })
             })
             .unwrap()
@@ -1251,8 +1298,9 @@ impl MedalConnection for Connection {
     fn get_task_by_id_complete(&self, task_id: i32) -> (Task, Taskgroup, Contest) {
         let query = "SELECT task.location, task.stars, taskgroup.id, taskgroup.name, taskgroup.active, contest.id,
                             contest.location, contest.filename, contest.name, contest.duration, contest.public,
-                            contest.start_date, contest.end_date, contest.min_grade, contest.max_grade,
-                            contest.requires_login, contest.requires_contest, contest.secret, contest.message
+                            contest.start_date, contest.end_date, contest.review_start_date, contest.review_end_date,
+                            contest.min_grade, contest.max_grade, contest.requires_login, contest.requires_contest,
+                            contest.secret, contest.message
                      FROM contest
                      JOIN taskgroup ON taskgroup.contest = contest.id
                      JOIN task ON task.taskgroup = taskgroup.id
@@ -1273,13 +1321,15 @@ impl MedalConnection for Connection {
                            public: row.get(10),
                            start: row.get(11),
                            end: row.get(12),
-                           min_grade: row.get(13),
-                           max_grade: row.get(14),
+                           review_start: row.get(13),
+                           review_end: row.get(14),
+                           min_grade: row.get(15),
+                           max_grade: row.get(16),
                            positionalnumber: None,
-                           requires_login: row.get(15),
-                           requires_contest: row.get(16),
-                           secret: row.get(17),
-                           message: row.get(18),
+                           requires_login: row.get(17),
+                           requires_contest: row.get(18),
+                           secret: row.get(19),
+                           message: row.get(20),
                            taskgroups: Vec::new() })
             })
             .unwrap()
@@ -1715,43 +1765,53 @@ impl MedalConnection for Connection {
         let duration = Duration::minutes(60);
         let then = now - duration;
 
+        // Zeit: 26,800 ms
         let query = "SELECT count(*)
                      FROM session
                      WHERE last_activity > $1;";
         let n_asession: i64 = self.query_map_one(query, &[&then], |row| row.get(0)).unwrap().unwrap();
 
+        // Zeit: 29,514 ms
         let query = "SELECT count(*)
                      FROM participation
                      WHERE start_date > $1;";
         let n_apart: i64 = self.query_map_one(query, &[&then], |row| row.get(0)).unwrap().unwrap();
 
+        // Zeit: 11,011 ms
         let query = "SELECT count(*)
                      FROM session;";
         let n_session: i64 = self.query_map_one(query, &[], |row| row.get(0)).unwrap().unwrap();
 
+        // Zeit: 26,959 ms
         let query = "SELECT count(*)
                      FROM session
                      WHERE oauth_foreign_id IS NOT NULL OR logincode IS NOT NULL;";
         let n_user: i64 = self.query_map_one(query, &[], |row| row.get(0)).unwrap().unwrap();
 
+        // Zeit: 25,129 ms
         let query = "SELECT count(*)
                      FROM session
                      WHERE oauth_foreign_id IS NOT NULL;";
         let n_pmsuser: i64 = self.query_map_one(query, &[], |row| row.get(0)).unwrap().unwrap();
 
+        // Zeit: 0,264 ms
         let query = "SELECT count(*)
                      FROM session
                      WHERE is_teacher = $1;";
         let n_teacher: i64 = self.query_map_one(query, &[&true], |row| row.get(0)).unwrap().unwrap();
 
+        // Zeit: 10,519 ms
         let query = "SELECT count(*)
                      FROM participation;";
         let n_part: i64 = self.query_map_one(query, &[], |row| row.get(0)).unwrap().unwrap();
 
-        let query = "SELECT count(*)
-                     FROM submission;";
-        let n_sub: i64 = self.query_map_one(query, &[], |row| row.get(0)).unwrap().unwrap();
+        // Zeit: 1205,003 ms (00:01,205)
+        // Currently disable to reduce load during contest
+        /*let query = "SELECT count(*)
+                     FROM submission;";*/
+        let n_sub: i64 = 0; /*self.query_map_one(query, &[], |row| row.get(0)).unwrap().unwrap();*/
 
+        // Zeit: 19,947 ms
         let query = "SELECT contest, count(*)
                      FROM participation
                      GROUP BY contest
